
// tslint:disable:variable-name Describing an API that's defined elsewhere.
// tslint:disable:no-any describes the API as best we are able today

import {html, css, LitElement, TemplateResult} from 'lit-element';

import {AmfHelperMixin} from '@api-components/amf-helper-mixin/amf-helper-mixin.js';

export {clearCache};
declare function clearCache(): void;

export {ApiAuthorization};


interface AuthorizationParams {
  headers: Object;
  params: Object;
  cookies: Object;
}
interface AuthorizationSettings {
  type: string;
  valid: boolean;
  settings: object;
}

/**
 * An element that renders various authorization methods.
 *
 * ## Development
 *
 * The element mixes in multimple mixins from `src/` directory.
 * Each mixin support an authorization method. When selection change (the `type`
 * property) a render function from correcponding mixin is called.
 */
declare class ApiAuthorization extends AmfHelperMixin(LitElement) {
  readonly styles: any;
  selected: number;
  /**
   * A security model generated by the AMF parser.
   */
  security: Array<Object>;
  methods: Array<Object>;
  compatibility: boolean;
  outlined: boolean;
  readOnly: boolean;
  disabled: boolean;
  invalid: boolean;
  redirectUri: string;
  httpMethod: string;
  requestUrl: string;
  requestBody: string;
  onchange: EventListener;
  readonly selectedMethods: Array<Object>;
  readonly selectedSchemes: Array<Object>;
  readonly settings: Array<Object>;

  constructor();
  serialize(): Array<AuthorizationSettings>;
  validate(): boolean;
  createAuthParams(): AuthorizationParams;
  forceAuthorization(validate: boolean): Promise<boolean>;
  render(): TemplateResult;
  clearCache(): void;
  _createSettings(target: Node): AuthorizationSettings;
  _processModel(): void;
  _applyModel(): void;
  _computeAuthMethods(securities: Array<Object>): Array<Object>;
  _listSchemeLabels(scheme: object): Array<String>;
  _selectionHandler(e: CustomEvent): void;
  _changeHandler(): void;
  _applyAuthParams(auth: AuthorizationSettings, target: AuthorizationParams): void;
  _applyBasicParams(auth: AuthorizationSettings, target: AuthorizationParams): void;
  _applyHeaderParams(auth: AuthorizationSettings, target: AuthorizationParams): void;
  _applyQueryParams(auth: AuthorizationSettings, target: AuthorizationParams): void;
  _applyPtParams(auth: AuthorizationSettings, target: AuthorizationParams): void;
  _applyOa2Params(auth: AuthorizationSettings, target: AuthorizationParams): void;
  _applyBearerParams(auth: AuthorizationSettings, target: AuthorizationParams): void;
  _applyApiKeyParams(auth: AuthorizationSettings, target: AuthorizationParams): void;
  _applyRamlCustomParams(auth: AuthorizationSettings, target: AuthorizationParams): void;
  _selectorTemplate(): TemplateResult|string;
  _singleItemTemplate(auth: object): TemplateResult;
  _selectorItem(item: object): TemplateResult;
  _methodsTemplate(): Array<TemplateResult|string>|string;
  _apiKeyTemplate(schema: object): TemplateResult;
  _renderMethod(type: string, scheme: object, renderTitle: boolean): TemplateResult|string;
  _methodTitleTemplate(scheme: object): TemplateResult|string;
  _basicAuthTemplate(security: object, renderTitle: boolean): TemplateResult;
  _digestAuthTemplate(security: object, renderTitle: boolean): TemplateResult;
  _passThroughAuthTemplate(security: object, renderTitle: boolean): TemplateResult;
  _ramlCustomAuthTemplate(security: object, renderTitle: boolean): TemplateResult;
  _bearerAuthTemplate(security: object, renderTitle: boolean): TemplateResult;
  _oa1AuthTemplate(security: object, renderTitle: boolean): TemplateResult;
  _oa2AuthTemplate(security: object, renderTitle: boolean): TemplateResult;
  _apiKeyTemplate(security: object, renderTitle: boolean): TemplateResult;
}
